@@ -20,13 +20,13 @@
 
 package Chart::Lines;
 
-use Chart::Base 2.3;
+use Chart::Base 2.4;
 use GD;
 use Carp;
 use strict;
 
 @Chart::Lines::ISA = qw(Chart::Base);
-$Chart::Lines::VERSION = '2.3';
+$Chart::Lines::VERSION = '2.4';
 
 #>>>>>>>>>>>>>>>>>>>>>>>>>>#
 #  public methods go here  #
@@ -122,8 +122,30 @@ sub _draw_data {
 	$y3 = $y1 - (($data->[$i][$j] - $mod) * $map);
 
         # now draw the line
-        $self->{'gd_obj'}->line($x2, $y2, $x3, $y3, gdBrushed);
-       
+        # ----------------
+        # stepline option added by G.ST. 2005/02
+        #----------------
+        if ($self->{'stepline'} =~ /^true$/i ) 
+        {
+           if ($self->{'stepline_mode'} =~ /^begin$/i ) 
+           {
+              $self->{'gd_obj'}->line($x2, $y2, $x3, $y2, gdBrushed);
+              $self->{'gd_obj'}->line($x3, $y2, $x3, $y3, gdBrushed);
+           }
+           else 
+           {
+              $self->{'gd_obj'}->line($x2, $y2, $x2, $y3, gdBrushed);
+              $self->{'gd_obj'}->line($x2, $y3, $x3, $y3, gdBrushed);
+           }
+        }
+        # -----------------------------------
+        # end stepline option
+        #------------------------------------
+        else
+        {
+            $self->{'gd_obj'}->line($x2, $y2, $x3, $y3, gdBrushed);
+        }
+        
         # set the flags, if the lines are out of the borders of the chart
         if ( ($data->[$i][$j] > $self->{'max_val'}) || ($data->[$i][$j-1] > $self->{'max_val'}) ) {
            $repair_top_flag = 1;
@@ -163,7 +185,7 @@ sub _draw_data {
   #repair the chart, if the lines are out of the borders of the chart
   if ($repair_top_flag) {
    #overwrite the ugly mistakes
-   $self->{'gd_obj'}->filledRectangle ($self->{'curr_x_min'}, 0,
+   $self->{'gd_obj'}->filledRectangle ($self->{'curr_x_min'}-($self->{'brush_size'}/2), 0,
 				$self->{'curr_x_max'}, $self->{'curr_y_min'}-1,
 				$self->_color_role_to_index('background'));
 
@@ -205,7 +227,7 @@ sub _draw_data {
   if ($repair_bottom_flag) {
 
    #overwrite the ugly mistakes
-   $self->{'gd_obj'}->filledRectangle ($self->{'curr_x_min'}, $self->{'curr_y_max'}+1,
+   $self->{'gd_obj'}->filledRectangle ($self->{'curr_x_min'}-($self->{'brush_size'}/2), $self->{'curr_y_max'}+1,
 				$self->{'curr_x_max'}, $abs_y_max,
 				$self->_color_role_to_index('background'));
     #save the actual x and y values
