@@ -1,25 +1,31 @@
-#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>#
-#  Chart::LinesPoints            #
-#                                #
-#  written by david bonner       #
-#  dbonner@cs.bu.edu             #
-#                                #
-#  maintained by the Chart Group #
-#  Chart@wettzell.ifag.de        #
-#                                #
-#                                #
-#  theft is treason, citizen     #
-#<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<#
-
+#====================================================================
+#  Chart::LinesPoints
+#                                
+#  written by david bonner        
+#  dbonner@cs.bu.edu              
+#
+#  maintained by the Chart Group
+#  Chart@wettzell.ifag.de
+#
+#---------------------------------------------------------------------
+# History:
+#----------
+# $RCSfile: LinesPoints.pm,v $ $Revision: 1.4 $ $Date: 2003/02/14 14:10:36 $
+# $Author: dassing $
+# $Log: LinesPoints.pm,v $
+# Revision 1.4  2003/02/14 14:10:36  dassing
+# First setup to cvs
+#
+#====================================================================
 package Chart::LinesPoints;
 
-use Chart::Base 2.0;
+use Chart::Base 2.4;
 use GD;
 use Carp;
 use strict;
 
 @Chart::LinesPoints::ISA = qw(Chart::Base);
-$Chart::LinesPoints::VERSION = '2.1';
+$Chart::LinesPoints::VERSION = '2.4';
 
 #>>>>>>>>>>>>>>>>>>>>>>>>>>#
 #  public methods go here  #
@@ -41,6 +47,8 @@ sub _draw_data {
   my ($i, $j, $color, $brush, $zero_offset, $delta_num);
   my $repair_top_flag = 0;
   my $repair_bottom_flag = 0;
+  my $diff;
+  
   # init the imagemap data field if they want it
   if ($self->{'imagemap'} =~ /^true$/i) {
     $self->{'imagemap_data'} = [];
@@ -50,8 +58,10 @@ sub _draw_data {
   # as the mapping constant
   $width = $self->{'curr_x_max'} - $self->{'curr_x_min'};
   $height = $self->{'curr_y_max'} - $self->{'curr_y_min'};
-  $delta = $width / $self->{'num_datapoints'};
-  $map = $height / ($self->{'max_val'} - $self->{'min_val'});
+  $delta = $width / ($self->{'num_datapoints'} > 0 ? $self->{'num_datapoints'} : 1);
+  $diff = ($self->{'max_val'} - $self->{'min_val'});
+  $diff = 1 if $diff == 0;
+  $map = $height / $diff;
 
   # get the base x-y values
   if ($self->{'xy_plot'} =~ /^true$/i) {
@@ -65,7 +75,7 @@ sub _draw_data {
     $mod = $self->{'min_val'};
   }
   elsif ($self->{'max_val'} <= 0) {
-    $y1 = $self->{'curr_y_min'};
+    $y1  = $self->{'curr_y_min'};
     $mod = $self->{'max_val'};
   }
   else {
@@ -78,6 +88,8 @@ sub _draw_data {
 
   #for a xy-plot, use this delta and maybe an offset for the zero-axes
   if ($self->{'xy_plot'} =~ /^true$/i ) {
+    $diff = ($self->{'x_max_val'} - $self->{'x_min_val'});
+    $diff = 1 if $diff == 0;
     $delta_num = $width / ($self->{'x_max_val'} - $self->{'x_min_val'});
 
     if ($self->{'x_min_val'} <= 0 && $self->{'x_max_val'} >= 0) {
@@ -127,7 +139,29 @@ sub _draw_data {
         }
 
         # draw the line
-	$self->{'gd_obj'}->line($x2, $y2, $x3, $y3, gdBrushed);
+        # ----------------
+        # stepline option added by G.ST. 2005/02
+        #----------------
+        if ($self->{'stepline'} =~ /^true$/i ) 
+        {
+           if ($self->{'stepline_mode'} =~ /^begin$/i ) 
+           {
+              $self->{'gd_obj'}->line($x2, $y2, $x3, $y2, gdBrushed);
+              $self->{'gd_obj'}->line($x3, $y2, $x3, $y3, gdBrushed);
+           }
+           else 
+           {
+              $self->{'gd_obj'}->line($x2, $y2, $x2, $y3, gdBrushed);
+              $self->{'gd_obj'}->line($x2, $y3, $x3, $y3, gdBrushed);
+           }
+        }
+        # -----------------------------------
+        # end stepline option
+        #------------------------------------
+        else
+	{
+            $self->{'gd_obj'}->line($x2, $y2, $x3, $y3, gdBrushed);
+        }
       }
     }
 
@@ -176,9 +210,10 @@ sub _draw_data {
   ($abs_x_max, $abs_y_max) = $self->{'gd_obj'}->getBounds();
 
   #repair the chart, if the lines are out of the borders of the chart
-  if ($repair_top_flag) {
+  if ($repair_top_flag) { 
    #overwrite the ugly mistakes
-   $self->{'gd_obj'}->filledRectangle ($self->{'curr_x_min'}, 0,
+#   $self->{'gd_obj'}->filledRectangle ($self->{'curr_x_min'}, 0,
+    $self->{'gd_obj'}->filledRectangle ($self->{'curr_x_min'}-($self->{'brush_size'}/2), 0,
 				$self->{'curr_x_max'}, $self->{'curr_y_min'}-2,
 				$self->_color_role_to_index('background'));
 
@@ -217,9 +252,10 @@ sub _draw_data {
    $self->{'curr_y_max'} = $t_y_max;
   }
 
-  if ($repair_bottom_flag) {
+  if ($repair_bottom_flag) { 
    #overwrite the ugly mistakes
-   $self->{'gd_obj'}->filledRectangle ($self->{'curr_x_min'}, $self->{'curr_y_max'}+1,
+# $self->{'gd_obj'}->filledRectangle ($self->{'curr_x_min'}, $self->{'curr_y_max'}+1,
+   $self->{'gd_obj'}->filledRectangle ($self->{'curr_x_min'}-($self->{'brush_size'}/2), $self->{'curr_y_max'}+1,
 				$self->{'curr_x_max'}, $abs_y_max,
 				$self->_color_role_to_index('background'));
 
@@ -271,9 +307,9 @@ sub _draw_data {
 
 ##  set the gdBrush object to trick GD into drawing fat lines
 sub _prepare_brush {
-  my $self = shift;
+  my $self  = shift;
   my $color = shift;
-  my $type = shift;
+  my $type  = shift;
   my ($radius, @rgb, $brush, $white, $newcolor);
 
   # get the rgb values for the desired color
