@@ -1,39 +1,46 @@
-#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>#
-#                                #
-#  Chart::Mountain               #
-#                                #
-#  Inspired by Chart::Lines      #
-#  by davidb bonner              #
-#  dbonner@cs.bu.edu             #
-#                                #
-#  Updated for                   #
-#  compatibility with            #
-#  changes to Chart::Base        #
-#  by peter clark                #
-#  ninjaz@webexpress.com         #
-#  written by david bonner       #
-#  dbonner@cs.bu.edu             #
-#                                #
-#  maintained by the Chart Group #
-#  Chart@wettzell.ifag.de        #
-#                                #
-#                                #
-#<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<#
-
+#====================================================================
+# 
+#  Chart::Mountain
+# 
+#  Inspired by Chart::Lines
+#  by davidb bonner 
+#  dbonner@cs.bu.edu
+# 
+#  Updated for 
+#  compatibility with 
+#  changes to Chart::Base
+#  by peter clark
+#  ninjaz@webexpress.com
+#
 # Copyright 1998, 1999 by James F. Miner.
 # All rights reserved. 
 # This program is free software; you can redistribute it 
 # and/or modify it under the same terms as Perl itself. 
+#
+#  maintained by the Chart Group
+#  Chart@wettzell.ifag.de
+#
+#---------------------------------------------------------------------
+# History:
+#----------
+# $RCSfile: Mountain.pm,v $ $Revision: 1.4 $ $Date: 2003/02/14 14:16:23 $
+# $Author: dassing $
+# $Log: Mountain.pm,v $
+# Revision 1.4  2003/02/14 14:16:23  dassing
+# First setup to cvs
+#
+#
+#====================================================================
 
 package Chart::Mountain;
 
-use Chart::Base 2.0;
+use Chart::Base 2.4;
 use GD;
 use Carp;
 use strict;
 
 @Chart::Mountain::ISA = qw ( Chart::Base );
-@Chart::Mountain::VERSION = '2.1';
+$Chart::Mountain::VERSION = '2.4';
 
 
 ##  Some Mountain chart details:
@@ -89,10 +96,6 @@ sub _find_y_range {
 	}
     }
 
-## new _find_y_scale does this:
-#   my $tmp = ($max) ? 10 ** (int (log ($max) / log (10))) : 10;
-#   $max = $tmp * (int ($max / $tmp) + 1);
-
     (0, $max);
 }
 
@@ -104,7 +107,7 @@ sub _draw_data {
     my @patterns = @{ $self->{'patterns'} || [] };
     
     # Calculate array of x pixel positions (@x).
-    my $x_step = ($self->{'curr_x_max'} - $self->{'curr_x_min'}) / $self->{'num_datapoints'};
+    my $x_step = ($self->{'curr_x_max'} - $self->{'curr_x_min'}) / ($self->{'num_datapoints'} > 0 ? $self->{'num_datapoints'} : 1);
     my $x_min = $self->{'curr_x_min'} + $x_step / 2;
     my $x_max = $self->{'curr_x_max'} - $x_step / 2;
     my @x = map { $_ * $x_step + $x_min } 0..$self->{'num_datapoints'}-1;
@@ -124,8 +127,9 @@ sub _draw_data {
 	for my $i (reverse 1..$#{$data}) { # bottom to top of chart
 	    my $datum = $data->[$i][$j];
 
-            #set the repair flag, if the datum is out of teh borders of the chart
-            if ($datum > $self->{'max_val'}) { $repair_top_flag = 1;}
+            #set the repair flag, if the datum is out of the borders of the chart
+            if ( defined $datum && $datum > $self->{'max_val'}) { $repair_top_flag = 1;}
+            
             
             if ( defined $datum && $datum >= 0 ) {
 		$sum += $datum;
@@ -144,6 +148,8 @@ sub _draw_data {
     while ( $x_begin <= $x_end && ! defined $y[-1]->[$x_begin] ) { $x_begin++ }
     while ( $x_begin <= $x_end && ! defined $y[-1]->[$x_end] ) { $x_end-- }
    
+    if ( $x_begin > $x_end ) { croak "Internal error: x_begin > x_end ($x_begin > $x_end)"; }
+    
     # For each dataset, generate a polygon for the dataset's area of the chart,
     # and fill the polygon with the dataset's color/pattern.
     
@@ -170,8 +176,7 @@ sub _draw_data {
 	# draw the polygon
 	my $color = $self->_color_role_to_index('dataset'.$dataset);
 	if ( $patterns[$dataset] ) {
-	    $self->{'gd_obj'}->filledPolygon($poly, $color) 
-		if $patterns[$dataset]->transparent >= 0;
+	    $self->{'gd_obj'}->filledPolygon($poly, $color) if $patterns[$dataset]->transparent >= 0;
 	    $self->{'gd_obj'}->setTile($patterns[$dataset]);
 	    $self->{'gd_obj'}->filledPolygon($poly, gdTiled);
 	}
