@@ -1,24 +1,33 @@
-#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>#
-#  Chart::Composite           #
-#                             #
-#  written by david bonner    #
-#  dbonner@cs.bu.edu          #
-#                             #
-#  maintained by peter clark  #
-#  ninjaz@webexpress.com      #
-#                             #
-#  theft is treason, citizen  #
-#<<<<<<<<<<<<<<<<<<<<<<<<<<<<<#
+#====================================================================
+#  Chart::Composite
+#
+#  written by david bonner
+#  dbonner@cs.bu.edu
+#
+#  maintained by the Chart Group
+#  Chart@wettzell.ifag.de
+#
+#
+#---------------------------------------------------------------------
+# History:
+#----------
+# $RCSfile: Composite.pm,v $ $Revision: 1.4 $ $Date: 2003/02/14 13:25:30 $
+# $Author: dassing $
+# $Log: Composite.pm,v $
+# Revision 1.4  2003/02/14 13:25:30  dassing
+# Circumvent division of zeros
+#
+#====================================================================
 
 package Chart::Composite;
 
-use Chart::Base;
+use Chart::Base 2.4;
 use GD;
 use Carp;
 use strict;
 
 @Chart::Composite::ISA = qw(Chart::Base);
-$Chart::Composite::VERSION = 2.0;
+$Chart::Composite::VERSION = '2.4';
 
 #>>>>>>>>>>>>>>>>>>>>>>>>>>#
 #  public methods go here  #
@@ -56,39 +65,51 @@ sub set {
 ##  had to override it to reassemble the @data array correctly
 sub imagemap_dump {
   my $self = shift;
-  my $ref = [];
   my ($i, $j);
-  my $data =[];
-  my ( $x, $y);
+  my @map;
+  my $dataset_count = 0;
+ 
   # croak if they didn't ask me to remember the data, or if they're asking
   # for the data before I generate it
   unless (($self->{'imagemap'} =~ /^true$/i) && $self->{'imagemap_data'}) {
     croak "You need to set the imagemap option to true, and then call the png method, before you can get the imagemap data";
   }
 
-  $data=$self->{'sub0'}->imagemap_dump();
-
-  ($x, $y)= @{$data->[1][0]};
-  print $y;
-  # make a copy for them, and reorder it
-  for $i (0..$#{$self->{'sub0'}->{'imagemap_data'}}) {
-    for $j (0..$#{$self->{'sub0'}->{'imagemap_data'}->[$i]}) {
-      $ref->[$self->{'composite_info'}->[0][1][$i]][$j] = 
-          [ @{ $self->{'sub0'}->{'imagemap_data'}->[$i][$j] } ];
+  #make a copy of the imagemap data
+  #this is the data of the first component
+  for $i (1..$#{$self->{'sub_0'}->{'imagemap_data'}}) {
+    for $j (0..$#{$self->{'sub_0'}->{'imagemap_data'}->[$i]}-1) {
+       $map[$i][$j] = \@{$self->{'sub_0'}->{'imagemap_data'}->[$i][$j]} ;
     }
+    $dataset_count++;
   }
-  for $i (0..$#{$self->{'sub1'}->{'imagemap_data'}}) {
-    for $j (0..$#{$self->{'sub1'}->{'imagemap_data'}->[$i]}) {
-      $ref->[$self->{'composite_info'}->[1][1][$i]][$j] = 
-          [ @{ $self->{'sub1'}->{'imagemap_data'}->[$i][$j] } ];
+  #and add the data of the second component
+  for $i (1..$#{$self->{'sub_1'}->{'imagemap_data'}}) {
+    for $j (0..$#{$self->{'sub_1'}->{'imagemap_data'}->[$i]}-1) {
+      $map[$i+$dataset_count][$j] = \@{$self->{'sub_1'}->{'imagemap_data'}->[$i][$j]} ;
     }
   }
+  
 
   # return their copy
-  return $ref;
+  return \@map;
+
 }
 
-
+sub __print_array {
+   my @a = @_;
+   my $i;
+   
+   my $li = $#a;
+   
+   $li++;
+   print STDERR "Anzahl der Elemente = $li\n"; $li--;
+   
+   for ($i=0; $i<=$li; $i++) {
+      print STDERR "\t$i\t$a[$i]\n";
+   }
+}
+   
 #>>>>>>>>>>>>>>>>>>>>>>>>>>>#
 #  private methods go here  #
 #<<<<<<<<<<<<<<<<<<<<<<<<<<<#
@@ -159,7 +180,8 @@ sub _split_data {
   $self->{'sub_0'} = ("Chart::".$types[0])->new();
   $self->{'sub_1'} = ("Chart::".$types[1])->new();
 
-  # set the options (set the min_val, max_val, and y_ticks
+  # set the options (set the min_val, max_val, brush_size, y_ticks,
+  # 
   # options intelligently so that the sub-objects don't get
   # confused)
   $self->{'sub_0'}->set (%{$self->{'opts'}});
@@ -182,6 +204,20 @@ sub _split_data {
   if ($self->{'opts'}{'y_ticks2'}) {
     $self->{'sub_1'}->set ('y_ticks' => $self->{'opts'}{'y_ticks2'});
   }
+  if ($self->{'opts'}{'brush_size1'}) {
+    $self->{'sub_0'}->set ('brush_size' => $self->{'opts'}{'brush_size1'});
+  } 
+  if ($self->{'opts'}{'brush_size2'}) {
+    $self->{'sub_1'}->set ('brush_size' => $self->{'opts'}{'brush_size2'});
+  }
+  
+  #  f_y_tick for left and right axis
+  if (defined ($self->{'opts'}{'f_y_tick1'})) {
+    $self->{'sub_0'}->set ('f_y_tick' => $self->{'opts'}{'f_y_tick1'});
+  }
+  if (defined ($self->{'opts'}{'f_y_tick2'})) {
+    $self->{'sub_1'}->set ('f_y_tick' => $self->{'opts'}{'f_y_tick2'});
+  }
 
   # replace the gd_obj fields
   $self->{'sub_0'}->{'gd_obj'} = $self->{'gd_obj'};
@@ -291,11 +327,17 @@ sub _draw_top_legend {
   my @labels = @{$self->{'legend_labels'}};
   my ($x1, $y1, $x2, $y2, $empty_width, $max_label_width, $cols, $rows, $color);
   my ($col_width, $row_height, $i, $j, $r, $c, $index, $x, $y, $sub, $w, $h);
+  my ($yh,$yi); #  for boxing legends
   my $font = $self->{'legend_font'};
   my (%colors, @datasets);
+  my $max_legend_example=0;
+  $yh=0;
 
   # copy the current boundaries into the sub-objects
   $self->_sub_update;
+  
+  # init the legend_example_height
+  $self->_legend_example_height_init;
 
 ## Make datasetI numbers match indexes of @{ $self->{'dataref'} }[1.....].
 #   # modify the dataset color table entries to avoid duplicating
@@ -377,7 +419,8 @@ sub _draw_top_legend {
           + ($rows * $row_height) + (2 * $self->{'legend_space'});
   $self->{'gd_obj'}->rectangle($x1, $y1, $x2, $y2, 
                                $self->_color_role_to_index('misc'));
-
+			       
+  $max_legend_example = $y2-$y1;
   # leave some space inside the legend
   $x1 += $self->{'legend_space'} + $self->{'text_space'};
   $x2 -= $self->{'legend_space'};
@@ -387,6 +430,8 @@ sub _draw_top_legend {
   # draw in the actual legend
   $r = 0; # current row
   $c = 0; # current column
+  $yi= 0; # current dataset
+   
   for $i (0..1) {
     for $j (0..$#{$self->{'component_datasets'}[$i]}) {
       # get the color
@@ -397,10 +442,35 @@ sub _draw_top_legend {
       $x = $x1 + ($col_width * $c);
       $y = $y1 + ($row_height * $r) + $h/2;
 
-      # draw the example line
+	  #  draw the example line if legend_example_height==1 or ==0
+	  if ($rows==1) {
+	  if ($self->{'legend_example_height'.$yi}<$max_legend_example) {
+	    $yh = $self->{'legend_example_height'.$yi};
+	    }
+	  else {
+	    $yh = $max_legend_example;
+	    }
+	   }
+	   else {
+	   if ($self->{'legend_example_height'.$yi}<$row_height) {
+	    $yh = $self->{'legend_example_height'.$yi};
+	    }
+	   else {
+	     $yh = $row_height;
+	     }
+	   } 
+	   $yi++;
+	if ($yh <= 1) {
       $self->{'gd_obj'}->line ($x, $y,
                                $x + $self->{'legend_example_size'}, $y,
 			       $color);
+	} else {
+	  #  draw the example bar if legend_example_height > 1
+	  $yh = int($yh / 2);
+      $self->{'gd_obj'}->filledRectangle ($x, $y-$yh,
+                               $x + $self->{'legend_example_size'}, $y+$yh,
+			       $color);
+	}
 
       # find the x-y coordinates for the beginning of the label
       $x += $self->{'legend_example_size'} + 2 * $self->{'text_space'};
@@ -417,7 +487,7 @@ sub _draw_top_legend {
       }
     }
   }
-      
+  
       
   # mark of the space used
   $self->{'curr_y_min'} += ($rows * $row_height)
@@ -433,12 +503,18 @@ sub _draw_right_legend {
   my $self = shift;
   my @labels = @{$self->{'legend_labels'}};
   my ($x1, $x2, $x3, $y1, $y2, $width, $color, $misccolor, $w, $h);
+  my ($yh)=0; # for boxing legend
   my $font = $self->{'legend_font'};
-  my (%colors, @datasets, $i);
-
+  my (%colors, @datasets, $i); 
+  my $max_legend_example = 0;
+  
+  
   # copy the current boundaries and colors into the sub-objects
   $self->_sub_update;
-
+  
+  # init the legend exapmle height
+  $self->_legend_example_height_init;
+  
 #   # modify the dataset color table entries to avoid duplicating
 #   # dataset colors (this limits the number of possible data sets
 #   # for each component to 8)
@@ -449,25 +525,29 @@ sub _draw_right_legend {
   # modify the dataset color table entries to avoid duplicating
   # dataset colors.
   my ($n0, $n1) = map { scalar @{ $self->{'composite_info'}[$_][1] } } 0..1;
+  
   for (0..$n1-1) {
     $self->{'sub_1'}{'color_table'}{'dataset'.$_} 
       = $self->{'color_table'}{'dataset'.($_+$n0)};
   }
-
+  
+  
+  
   # make sure we use the right colors for the legend
   @datasets = @{$self->{'composite_info'}[0][1]};
   $i = 0;
   for (0..$#datasets) {
-    $colors{$datasets[$_]-1} = $self->{'color_table'}{'dataset'.($i)};
+   $colors{$datasets[$_]-1} = $self->{'color_table'}{'dataset'.($_)};
     $i++;
   }
+   
   @datasets = @{$self->{'composite_info'}[1][1]};
   $i = 0;
   for (0..$#datasets) {
     $colors{$datasets[$_]-1} = $self->{'color_table'}{'dataset'.($i+$n0)};
     $i++;
   }
-
+      
   # make sure we're using a real font
   unless ((ref ($font)) eq 'GD::Font') {
     croak "The subtitle font you specified isn\'t a GD Font object";
@@ -475,7 +555,7 @@ sub _draw_right_legend {
 
   # get the size of the font
   ($h, $w) = ($font->height, $font->width);
-
+ 
   # get the miscellaneous color
   $misccolor = $self->_color_role_to_index('misc');
 
@@ -499,22 +579,37 @@ sub _draw_right_legend {
   $y1 += $self->{'legend_space'} + $self->{'text_space'};
 
   # now draw the actual legend
-  for (0..$#labels) {
+  for (0..$#labels) { 
     # get the color
     $color = $colors{$_};
-
+    
+    # find the max_legend_example
+    $max_legend_example = $self->{'legend_space'}+$h; 
+   
     # find the x-y coords
     $x2 = $x1;
     $x3 = $x2 + $self->{'legend_example_size'};
     $y2 = $y1 + ($_ * ($self->{'text_space'} + $h)) + $h/2;
 
-    # do the line first
-    $self->{'gd_obj'}->line ($x2, $y2, $x3, $y2, $color);
+	# draw the example line if legend_example_height==1 or ==0
+	if ($self->{'legend_example_height'.$_}<$max_legend_example) {
+	   $yh = $self->{'legend_example_height'.$_};
+	   }
+	else {
+	   $yh = $max_legend_example;
+	   }   
+	if ($yh <= 1) {
+      $self->{'gd_obj'}->line ($x2, $y2, $x3, $y2, $color);
+	} else {
+	  $yh = int($yh / 2);
+	  $self->{'gd_obj'}->filledRectangle ($x2, $y2-$yh, $x3, $y2+$yh, $color);
+	}
 
     # now the label
     $x2 = $x3 + (2 * $self->{'text_space'});
     $y2 -= $h/2;
     $self->{'gd_obj'}->string ($font, $x2, $y2, $labels[$_], $color);
+   
   }
 
   # mark off the used space
@@ -529,12 +624,16 @@ sub _draw_left_legend {
   my $self = shift;
   my @labels = @{$self->{'legend_labels'}};
   my ($x1, $x2, $x3, $y1, $y2, $width, $color, $misccolor, $w, $h);
+  my $yh; # for boxing legend
   my $font = $self->{'legend_font'};
   my (%colors, @datasets, $i);
+  my $max_legend_example=0;
  
   # copy the current boundaries and colors into the sub-objects
   $self->_sub_update;
- 
+  
+  # init the legend_example height
+  $self->_legend_example_height_init;
 #   # modify the dataset color table entries to avoid duplicating
 #   # dataset colors (this limits the number of possible data sets
 #   # for each component to 8)
@@ -601,14 +700,29 @@ sub _draw_left_legend {
   for (0..$#labels) {
     # get the color
     $color = $colors{$_};
- 
+   
+    # find the max_legend_example    
+    $max_legend_example = $self->{'legend_space'} + $h;
+    
     # find the x-y coords
     $x2 = $x1;
     $x3 = $x2 + $self->{'legend_example_size'};
     $y2 = $y1 + ($_ * ($self->{'text_space'} + $h)) + $h/2;
  
-    # do the line first
-    $self->{'gd_obj'}->line ($x2, $y2, $x3, $y2, $color);
+	# draw the example line if legend_example_height==1 or ==0
+	if ($self->{'legend_example_height'.$_}<$max_legend_example) {
+	   $yh = $self->{'legend_example_height'.$_};
+	   }
+	else {
+	   $yh = $max_legend_example;
+	   }   
+	if ($yh <= 1) {
+      $self->{'gd_obj'}->line ($x2, $y2, $x3, $y2, $color);
+	} else {
+	  # draw the example bar if legend_example_height > 1
+	  $yh = int($yh / 2);
+      $self->{'gd_obj'}->filledRectangle ($x2, $y2-$yh, $x3, $y2+$yh, $color);
+	}
  
     # now the label
     $x2 = $x3 + (2 * $self->{'text_space'});
@@ -630,12 +744,17 @@ sub _draw_bottom_legend {
   my @labels = @{$self->{'legend_labels'}};
   my ($x1, $y1, $x2, $y2, $empty_width, $max_label_width, $cols, $rows, $color);
   my ($col_width, $row_height, $i, $j, $r, $c, $index, $x, $y, $sub, $w, $h);
+  my ($yh,$yi); # for boxing legend
   my $font = $self->{'legend_font'};
   my (%colors, @datasets);
+  my $max_legend_example=0;
+  $yh=0;
 
   # copy the current boundaries and colors into the sub-objects
   $self->_sub_update;
-
+  # init the legend example height
+  $self->_legend_example_height_init;
+  
 #   # modify the dataset color table entries to avoid duplicating
 #   # dataset colors (this limits the number of possible data sets
 #   # for each component to 8)
@@ -671,6 +790,7 @@ sub _draw_bottom_legend {
 
   # get the size of the font
   ($h, $w) = ($font->height, $font->width);
+  
 
   # figure out how many columns we can fit
   $x1 = $self->{'curr_x_min'} + $self->{'graph_border'}
@@ -699,7 +819,7 @@ sub _draw_bottom_legend {
   $rows = int ($self->{'num_datasets'} / $cols);
   unless (($self->{'num_datasets'} % $cols) == 0) {
     $rows++;
-  }
+    }
   unless ($rows) {
     $rows = 1;
   }
@@ -711,6 +831,10 @@ sub _draw_bottom_legend {
   $y2 = $self->{'curr_y_max'};
   $self->{'gd_obj'}->rectangle($x1, $y1, $x2, $y2, 
                                $self->_color_role_to_index('misc'));
+			       
+  # get the max_legend_example_height			       
+  $max_legend_example = $y2-$y1;
+ 		       
   $x1 += $self->{'legend_space'} + $self->{'text_space'};
   $x2 -= $self->{'legend_space'};
   $y1 += $self->{'legend_space'} + $self->{'text_space'};
@@ -719,17 +843,46 @@ sub _draw_bottom_legend {
   # draw in the actual legend
   $r = 0;
   $c = 0;
-  for $i (0..1) {
-    for $j (0..$#{$self->{'component_datasets'}[$i]}) {
+  $yi= 0; # current dataset
+    for $i (0..1) {
+      for $j (0..$#{$self->{'component_datasets'}[$i]}) {
       $color = $self->{'sub_'.$i}->{'color_table'}{'dataset'.$j};
       $index = $self->{'component_datasets'}[$i][$j] - 1;
 
       $x = $x1 + ($col_width * $c);
       $y = $y1 + ($row_height * $r) + $h/2;
-      $self->{'gd_obj'}->line ($x, $y,
-                               $x + $self->{'legend_example_size'}, $y,
-			       $color);
-
+	  
+	  #  draw the example line if legend_example_height==1 or ==0
+	  if ($rows == 1) {
+	  if ($self->{'legend_example_height'.$yi} < $max_legend_example) {
+	    $yh = $self->{'legend_example_height'.$yi};
+	    }
+	  else {
+	    $yh = $max_legend_example;
+	    }
+	  }
+	  else {
+	  if ($self->{'legend_example_height'.$yi} < $row_height) {
+	     $yh = $self->{'legend_example_height'.$yi};
+	     }
+	  else {
+	     $yh = $row_height;
+	     }
+         }    
+	 $yi++;
+	 if ($yh <= 1) {
+          $self->{'gd_obj'}->line ($x, $y,
+		                         $x + $self->{'legend_example_size'}, $y,
+								 $color);
+	  } 
+	  else {
+	    # draw the example bar if legend_example_height > 1
+	    $yh = int($yh / 2);
+        $self->{'gd_obj'}->filledRectangle ($x, $y-$yh,
+				            $x + $self->{'legend_example_size'}, $y+$yh,
+							   $color);
+  	  }
+ 
       $x += $self->{'legend_example_size'} + 2 * $self->{'text_space'};
       $y -= $h/2;
       $self->{'gd_obj'}->string($font, $x, $y, 
@@ -741,9 +894,7 @@ sub _draw_bottom_legend {
 	$c++;
       }
     }
-  }
-      
-      
+  }   
   # mark of the space used
   $self->{'curr_y_max'} -= ($rows * $row_height)
   			      + $self->{'text_space'}
@@ -822,14 +973,14 @@ sub _draw_x_ticks {
 #   $x2 = $self->{'curr_x_max'} - ($w * $self->{'y_tick_label_length2'})
 #          - (2 * $self->{'text_space'}) - $self->{'tick_len'};
   $x1 = $self->{'curr_x_min'} + ($w * $self->{'y_tick_label_length1'})
-         + (3 * $self->{'text_space'}) + $self->{'tick_len'};
+        + (3 * $self->{'text_space'}) + $self->{'tick_len'};
   $x2 = $self->{'curr_x_max'} - ($w * $self->{'y_tick_label_length2'})
-         - (3 * $self->{'text_space'}) - $self->{'tick_len'};
+        - (3 * $self->{'text_space'}) - $self->{'tick_len'};
   $y1 = $self->{'curr_y_max'} - $h - $self->{'text_space'};
 
   # get the delta value, figure out how to draw the labels
   $width = $x2 - $x1;
-  $delta = $width / $self->{'num_datapoints'};
+  $delta = $width / ( $self->{'num_datapoints'} > 0 ? $self->{'num_datapoints'} : 1 );
   if ($delta <= ($self->{'x_tick_label_length'} * $w)) {
     unless ($self->{'x_ticks'} =~ /^vertical$/i) {
       $self->{'x_ticks'} = 'staggered';
@@ -839,7 +990,8 @@ sub _draw_x_ticks {
   # now draw the labels
   if ($self->{'x_ticks'} =~ /^normal$/i) { # normal ticks
     if ($self->{'skip_x_ticks'}) {
-      for (0..int(($self->{'num_datapoints'}-1)/$self->{'skip_x_ticks'})) {
+      for (0..int(($self->{'num_datapoints'}-1)/$self->{'skip_x_ticks'})) 
+      {
         $x2 = $x1 + ($delta/2) + ($delta*($_*$self->{'skip_x_ticks'})) 
 	      - ($w*length( $self->{'f_x_tick'}->($data->[0][$_* $self->{'skip_x_ticks'}]))) / 2;
         $self->{'gd_obj'}->string($font, $x2, $y1, 
@@ -854,8 +1006,10 @@ sub _draw_x_ticks {
                                   $self->{'f_x_tick'}->($data->[0][$_]), $textcolor);
       }
     }
-    else {
-      for (0..$self->{'num_datapoints'}-1) {
+    else 
+    {
+      for (0..$self->{'num_datapoints'}-1) 
+      {
         $x2 = $x1 + ($delta/2) + ($delta*$_) - ($w*length($self->{'f_x_tick'}->($data->[0][$_]))) / 2;
         $self->{'gd_obj'}->string($font, $x2, $y1, $self->{'f_x_tick'}->($data->[0][$_]), $textcolor);
       }
@@ -908,7 +1062,7 @@ sub _draw_x_ticks {
   }
   elsif ($self->{'x_ticks'} =~ /^vertical$/i) { # vertical ticks
     $y1 = $self->{'curr_y_max'} - $self->{'text_space'};
-    if ($self->{'skip_x_ticks'}) {
+    if ( defined($self->{'skip_x_ticks'}) && $self->{'skip_x_ticks'} > 1) {
       for (0..int(($self->{'num_datapoints'}-1)/$self->{'skip_x_ticks'})) {
         $x2 = $x1 + ($delta/2) + ($delta*($_*$self->{'skip_x_ticks'})) - $h/2;
         $y2 = $y1 - (($self->{'x_tick_label_length'} 
@@ -1049,6 +1203,7 @@ sub _draw_data {
   }
 
   # now let the component modules go to work
+  
   $self->{'sub_0'}->_draw_data;
   $self->{'sub_1'}->_draw_data;
       
@@ -1101,5 +1256,47 @@ sub _draw_y2_grid_lines {
 	return;
 }
 
+
+# init the legend_example_height_values
+sub _legend_example_height_init {
+  my $self = shift;
+  my $a = $self->{'num_datasets'};
+  my ($b, $e) =(0,0);
+  my $bis='..';
+  
+      
+  if ($self->{'legend_example_height'} =~ /^false$/i ) {
+  
+    for my $i (0..$a) {
+    $self->{'legend_example_height'.$i} = 1;
+    }
+  }
+ 
+  if ($self->{'legend_example_height'} =~ /^true$/i ) { 
+ 
+   for my $i (0..$a) {
+   
+   if (defined($self->{'legend_example_height'.$i})) { }
+   else { ($self->{'legend_example_height'.$i}) = 1;}   
+  
+   }
+
+   for  $b (0..$a) {
+    for  $e (0..$a) {
+      my $anh = sprintf($b.$bis.$e);
+       if (defined($self->{'legend_example_height'.$anh})) {
+        if ($b>$e) {croak "Please reverse the datasetnumber in legend_example_height\n";}
+       	  for (my $n=$b;$n<=$e;$n++) {
+	    $self->{'legend_example_height'.$n} = $self->{'legend_example_height'.$anh};
+	 }
+        }
+       }
+    }
+   }
+ 
+  
+  
+}     
+
 ## be a good module and return 1
 1;
